from pandas import DataFrame, read_csv
from sklearn.preprocessing import LabelEncoder
from sklearn.model_selection import KFold
from imblearn.combine import SMOTEENN


def construct_dataframes(train, test):
    file_list = [train, test]
    df_dict = {}
    for file in file_list:
        df_dict[file] = read_csv(filepath_or_buffer=file)
    df_list = list(df_dict.values())
    return df_list


def rename_columns(df_list) -> DataFrame:
    for df in df_list:
        df.columns = df.columns.str.lower()
    return df_list


def process_null_values(df_list):
    drop_columns = ["tipo_marchas", "descuento", "ciudad"]
    fill_columns = ["asientos", "motor_cc", "potencia"]
    for df in df_list:
        for column in fill_columns:
            if column == "asientos":
                df[column].fillna(value=df[column].median(), inplace=True)
            else:
                df[column].fillna(
                    value=df[column].str.extract("(\d+)").mean(), inplace=True
                )
        df.drop(columns=drop_columns, inplace=True)
        df.dropna(inplace=True)
    return df_list


def encode_columns(df_list):
    label_encoder = LabelEncoder()
    files = [
        "ao",
        "asientos",
        "combustible",
        "consumo",
        "kilometros",
        "mano",
        "motor_cc",
        "nombre",
        "potencia",
    ]
    for data in files:
        for df in df_list:
            label = label_encoder.fit(read_csv("data/" + data + ".csv", squeeze=True))
            if data == "ao":
                df["año"] = label.transform(df["año"])
            else:
                df[data] = label.transform(df[data])
    return df_list


def split_data_target(df, dataset):
    if dataset == "data":
        df.drop(columns="id", inplace=True)
        data = df.drop(columns=["precio_cat"])
        target = df["precio_cat"]
    else:
        data = df.drop(columns=["id"])
        target = df["id"]
    return data, target


def balance_training_data(df):
    smote_enn = SMOTEENN(random_state=42)
    data, target = split_data_target(df=df, dataset="data")
    balanced_data, balanced_target = smote_enn.fit_resample(data, target)
    balanced_data_df = DataFrame(
        balanced_data, columns=df.columns.difference(["precio_cat"])
    )
    balanced_target_df = DataFrame(balanced_target, columns=["precio_cat"])
    return balanced_data_df, balanced_target_df


def split_k_sets(df):
    k_fold = KFold(shuffle=True, random_state=42)
    return k_fold.split(df)


def parse_data(train, test):
    df_list = construct_dataframes(train=train, test=test)
    renamed_df_list = rename_columns(df_list)
    processed_df_list = process_null_values(renamed_df_list)
    encoded_df_list = encode_columns(processed_df_list)
    train_data, train_target = balance_training_data(encoded_df_list[0])
    test_data, test_ids = split_data_target(encoded_df_list[1], dataset="test")
    return train_data, train_target, test_data, test_ids
